
from importlib import import_module
import inspect
import logging
import sys

# from dmzb.slang.pidgin.unifier import Unifier


class CommandException(Exception): pass

class Command(object):
    
    command_language = '''
        channel: describe channel named <name>
        channel: describe channel number <number>
        channel: list channels
        config: set database config <server> <name> <user> <password>
        file: list files for [recording] group <group_name>
        file: list files with no recording
        file: list files with recordings marked for delete
        file: list storage groups
        help: help
        help: help [with] [command] <command>
        recording: describe recording with program id <program_id>
        recording: list recordings
        recording: list recordings marked for delete
        recording: list recordings for channel number <channel_number>
    '''
    
    imported_modules = {}
    
    @classmethod
    def dispatch(cls, unifier):
        command = cls.get_command(unifier.predicate.name, unifier)
        command.execute()
    
    @classmethod
    def get_command(cls, name, unifier):
        try:
            if name not in cls.imported_modules: 
                cls.imported_modules[name] = import_module('mythos.command.' + name)
            full_name = name[0].upper() + name[1:] + 'Command'
            return getattr(cls.imported_modules[name], full_name)(unifier)
        except Exception, e:
            raise CommandException("Failed to instantiate command: %s - %s" % (name, e))
    
    @classmethod
    def get_help(cls):
        raise NotImplementedError('Must be implemented by subclass')
    
    @classmethod
    def get_name(cls):
        raise NotImplementedError('Must be implemented by subclass')
    
    def __init__(self, unifier):
        self.unifier = unifier
    
    def __str__(self):
        return "Command: " + " ".join([str(token) for token in self.unifier.tokens])
    
    def execute(self):
        raise NotImplementedError('Must be implemented by subclass')
    
    