from mythos import debug_trace, logger, UnconfiguredException
from mythos.command import Command, CommandException
from mythos.model import File, Recording, StorageGroup
from mythos.session import new_query

from os import listdir, path

_interesting_extensions = ['.mpg']

class FileCommand(Command):
    @classmethod
    def get_help(cls):
        return '''
Help is TBD
'''

    def files_for_group_name(self, group_name):
        group = new_query(StorageGroup, {StorageGroup.group_name: group_name}).first()
        if not group:
            print 'No storage group: %s' % group_name
            return []
        return self.files_for_groups([group])
    
    def files_for_groups(self, groups):
        files = []
        for group in groups:
            if path.exists(group.directory_name):
                if path.isdir(group.directory_name):
                    for child in listdir(group.directory_name):
                        if path.isfile(child):
                            if any([child.endswith(extension) for extension in _interesting_extensions]):
                                recording = new_query(Recording, {Recording.file_name: path.basename(child)}).first()
                                files.append(File(child, recording, group))
                else:
                    print "The path: %s is not a directory." % group.directory_name
            else:
                print "The directory %s does not exist on this host." % group.directory_name
        return files
        
    @classmethod
    def get_name(cls):
        return 'file'
    
    def execute(self):
        logger().debug("Executing file command: " + str(self))
        try:
            if 'group' in self.unifier.tokens:
                if 'group_name' in self.unifier.variables:
                    group_name = self.unifier.variables['group_name']
                    files = self.files_for_group_name(group_name)
                    File.show_list(files)
                else:
                    raise ValueError("No group_name parameter")
            elif 'groups' in self.unifier.tokens:
                groups = new_query(StorageGroup).all()
                StorageGroup.show_list(groups)
            elif self.unifier.has_subsequence(['with', 'no', 'recording']):
                groups = new_query(StorageGroup).all()
                files = self.files_for_groups(groups)
                files = filter(lambda file: file.recording, files)
                File.show_list(files)
            elif self.unifier.has_subsequence(['marked', 'for', 'delete']):
                groups = new_query(StorageGroup).all()
                files = self.files_for_groups(groups)
                files = filter(lambda file: file.recording and file.recording.delete_pending, files)
                File.show_list(files)
            else:
                raise NotImplementedError()
        except UnconfiguredException, u:
            raise u
        except Exception, e:
            debug_trace("file command")
            raise CommandException("Error processing file command: " + str(self), e)