
from mythos import debug_trace, logger, Config, MythosException, UnconfiguredException
from sqlalchemy.engine import create_engine
from sqlalchemy.engine.url import URL
from sqlalchemy.orm import sessionmaker


class SessionException(MythosException): pass

class Session:
    engine = None
    handle = None
    session = None
    
    @classmethod
    def get_engine(cls):
        if not cls.engine:
            handle = cls.get_handle()
            try:
                logger().debug("Creating database engine: " + str(handle))
                url = URL(handle['driver'],
                    database=handle['database_name'],
                    host=handle['server'],
                    username=handle['user_name'],
                    password=handle['password']
                )
                cls.engine = create_engine(url)
            except UnconfiguredException, u:
                raise u
            except Exception, e:
                debug_trace("get_engine")
                raise SessionException("Error creating engine", e)
        return cls.engine

    @classmethod
    def get_handle(cls):
        if not cls.handle:
            cls.handle = Config.load_config()
        return cls.handle
    
    @classmethod
    def get_session(cls):
        if not cls.session:
            try:
                logger().debug("Connecting to database: " + str(cls.get_handle()))
                cls.session = sessionmaker(bind=cls.get_engine())()
                logger().debug("Connected: " + str(cls.get_handle))
            except UnconfiguredException, u:
                raise u
            except Exception, e:
                debug_trace("get_session")
                raise SessionException("Error creating session", e)
        return cls.session
    
def new_query(xclass, constraints={}, joins=[]):
    query = Session.get_session().query(xclass)
    for join in joins:
        query = query.join(join)
    for key, value in constraints.items():
        query = query.filter(key == value)
    return query

